#ifndef _ASM_SMP_BALANCE_H
#define _ASM_SMP_BALANCE_H

#define ARCH_HAS_LOAD_BALANCE

/*
 * Find any idle processor package (i.e. both virtual processors are idle)
 */

static inline int find_idle_package(int this_cpu)
{
	int i = this_cpu + 1;

	if (i == smp_num_cpus)
		i = 0;

	while (i != this_cpu) {
		int physical, sibling;
		physical = cpu_logical_map(i);
		sibling = cpu_sibling_map[physical];

		if (i++ == smp_num_cpus)
			i = 0;
		if (idle_cpu(physical) && idle_cpu(sibling))
			return physical;
	}
	return -1;                      /* not found */
}

static inline int arch_load_balance(int this_cpu, int idle)
{
	/* Special hack for hyperthreading */
       if (smp_num_siblings > 1 && idle && !idle_cpu(cpu_sibling_map[this_cpu])) {
               int found;
               struct runqueue *rq_target;

               if ((found = find_idle_package(this_cpu)) >= 0 ) {
                       rq_target = cpu_rq(found);
                       resched_task(rq_target->idle);
                       return 1;
               }
       }
       return 0;
}
#endif
